/******************************************************************************/
// Bullfrog Engine Emulation Library - for use to remake classic games like
// Syndicate Wars, Magic Carpet or Dungeon Keeper.
/******************************************************************************/
/** @file bflib_render_gtblock.c
 *     Rendering function gtblock_draw() for drawing 3D view elements.
 * @par Purpose:
 *     Function for rendering 3D elements.
 * @par Comment:
 *     Go away from here, you bad optimizer! Do not compile this with optimizations.
 * @author   Tomasz Lis
 * @date     05 Oct 2010 - 11 Nov 2010
 * @par  Copying and copyrights:
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 */
/******************************************************************************/
#include "pre_inc.h"
#include "bflib_render.h"

#include "globals.h"
#include "bflib_basics.h"
#include "bflib_memory.h"
#include "bflib_video.h"
#include "bflib_sprite.h"
#include "bflib_vidraw.h"
#include "post_inc.h"

/******************************************************************************/
static unsigned char *gtblock_screen_addr;
static long gtblock_clip_width;
static long gtblock_clip_height;
static long gtblock_screen_width;
/******************************************************************************/
long const gtblock_arr08[] = { 0x08, 0x10, 0x20, };
long const gtblock_arr07[] = { 0x10, 0x20, };
long const gtblock_arr06[] = {
    0,31,7168,7199,0,7936,28,7964,
    0,31,7168,7199,0,7936,28,7964,
    0,31,7680,7711,0,7936,30,7966,
    0,31,7680,7711,0,7936,30,7966,
    0,31,7936,7967,0,7936,31,7967,
    0,31,7936,7967,0,7936,31,7967,
    };
long const gtblock_arr04[] = {
    1024,1024,-1024,-1024,4,4,-4,-4,
    1024,1024,-1024,-1024,4,4,-4,-4,
    512,512,-512,-512,2,2,-2,-2,
    512,512,-512,-512,2,2,-2,-2,
    256,256,-256,-256,1,1,-1,-1,
    256,256,-256,-256,1,1,-1,-1,
    };
long const gtblock_arr05[] = {
    2,-2,2,-2,512,-512,512,-512,
    1,-1,1,-1,256,-256,256,-256,
    2,-2,2,-2,512,-512,512,-512,
    1,-1,1,-1,256,256,-256,-256,
    2,-2,2,-2,512,-512,512,-512,
    1,-1,1,-1,256,-256,256,-256,
    };
long const gtblock_arr01[] = {
    12,12,12,12,12,12,12,12,
    11,11,11,11,11,11,11,11,
    12,12,12,12,12,12,12,12,
    11,11,11,11,11,11,11,11,
    12,12,12,12,12,12,12,12,
    11,11,11,11,11,11,11,11,
    };
long const gtblock_arr03[] = {
    9,9,9,9,9,9,9,9,
    8,8,8,8,8,8,8,8,
    8,8,8,8,8,8,8,8,
    7,7,7,7,7,7,7,7,
    7,7,7,7,7,7,7,7,
    6,6,6,6,6,6,6,6,
    };
long const gtblock_arr02[] = { 
    13,13,13,13,13,13,13,13,
    13,13,13,13,13,13,13,13,
    12,12,12,12,12,12,12,12,
    12,12,12,12,12,12,12,12,
    11,11,11,11,11,11,11,11,
    11,11,11,11,11,11,11,11,
    };
/******************************************************************************/
unsigned char *LOC_gtblock_screen_addr;
long LOC_gtblock_clip_width;
long LOC_gtblock_clip_height;
long LOC_gtblock_screen_width;
/******************************************************************************/
void gtblock_set_clipping_window(unsigned char *screen_addr, long clip_width, long clip_height, long screen_width)
{
    gtblock_screen_addr = screen_addr;
    gtblock_clip_width = clip_width;
    gtblock_clip_height = clip_height;
    gtblock_screen_width = screen_width;
}
/******************************************************************************/
void gtblock_draw(struct GtBlock *gtb)
{
    LOC_gtblock_screen_addr = gtblock_screen_addr;
    LOC_gtblock_clip_width = gtblock_clip_width;
    LOC_gtblock_clip_height = gtblock_clip_height;
    LOC_gtblock_screen_width = gtblock_screen_width;

#if __GNUC__
    asm volatile (" \
var_44  =   -0x44\n \
var_40  =   -0x40\n \
var_3C  =   -0x3C\n \
var_38  =   -0x38\n \
var_34  =   -0x34\n \
var_30  =   -0x30\n \
var_2C  =   -0x2C\n \
var_28  =   -0x28\n \
var_24  =   -0x24\n \
\n \
    pusha   \n \
    movl    %%eax,%%ebp\n \
    movl    0x1C(%%ebp),%%edi\n \
    movl    0x20(%%ebp),%%eax\n \
    shll    $1,%%eax\n \
    addl    %%eax,%%edi\n \
    movl    0x24(%%ebp),%%eax\n \
    shll    $2,%%eax\n \
    addl    %%eax,%%edi\n \
    movl    0x28(%%ebp),%%eax\n \
    shll    $3,%%eax\n \
    addl    %%eax,%%edi\n \
    movl    0x2C(%%ebp),%%eax\n \
    shll    $4,%%eax\n \
    addl    %%eax,%%edi\n \
    pushl   %%edi\n \
    movl    0x28(%%ebp),%%eax\n \
    shll    $2,%%eax\n \
    pushl   %%eax\n \
    movl    0x0C(%%ebp),%%eax\n \
    shll    $0x10,%%eax\n \
    pushl   %%eax\n \
    movl    0x10(%%ebp),%%eax\n \
    subl    0x0C(%%ebp),%%eax\n \
    movl    _gtblock_arr01(,%%edi,4),%%ecx\n \
    shll    %%cl,%%eax\n \
    pushl   %%eax\n \
    movl    0x14(%%ebp),%%eax\n \
    subl    0x0C(%%ebp),%%eax\n \
    movl    _gtblock_arr02(,%%edi,4),%%ecx\n \
    shll   %%cl,%%eax\n \
    pushl   %%eax\n \
    movl    0x18(%%ebp),%%eax\n \
    addl    0x0C(%%ebp),%%eax\n \
    subl    0x10(%%ebp),%%eax\n \
    subl    0x14(%%ebp),%%eax\n \
    movl    _gtblock_arr03(,%%edi,4),%%ecx\n \
    shll   %%cl,%%eax\n \
    pushl   %%eax\n \
    movl    _gtblock_arr04(,%%edi,4),%%eax\n \
    pushl   %%eax\n \
    movl    _gtblock_arr05(,%%edi,4),%%eax\n \
    pushl   %%eax\n \
    movl    _gtblock_arr06(,%%edi,4),%%ebx\n \
    addl    0(%%ebp),%%ebx\n \
    movl    0x28(%%ebp),%%eax\n \
    movl    _gtblock_arr07(,%%eax,4),%%eax\n \
    movl    0x2C(%%ebp),%%ecx\n \
    movl    _gtblock_arr08(,%%ecx,4),%%ecx\n \
    movl    8(%%ebp),%%esi\n \
    orl %%esi,%%esi\n \
    jns loc_786668\n \
    addl    %%esi,%%ecx\n \
    jz  locret1\n \
    js  locret1\n \
    negl    %%esi\n \
    movl    0x40+var_34(%%esp),%%edx\n \
    imull   %%esi,%%edx\n \
    addl    %%edx,0x40+var_2C(%%esp)\n \
    movl    0x40+var_38(%%esp),%%edx\n \
    imull   %%esi,%%edx\n \
    addl    %%edx,0x40+var_30(%%esp)\n \
    movl    0x40+var_3C(%%esp),%%edx\n \
    imull   %%esi,%%edx\n \
    addl    %%edx,%%ebx\n \
    xorl    %%esi,%%esi\n \
\n \
loc_786668:         #9D\n \
    movl    %%esi,%%edx\n \
    addl    %%ecx,%%edx\n \
    cmpl    _LOC_gtblock_clip_height,%%edx\n \
    jle loc_786688\n \
    subl    %%edx,%%ecx\n \
    addl    _LOC_gtblock_clip_height,%%ecx\n \
    jz  locret1\n \
    js  locret1\n \
\n \
loc_786688:         #DA\n \
    movl    4(%%ebp),%%edi\n \
    orl %%edi,%%edi\n \
    jns loc_7866D0\n \
    addl    %%edi,%%eax\n \
    jz  locret1\n \
    js  locret1\n \
    movl    $8,%%edx\n \
    orl %%edx,0x40+var_28(%%esp)\n \
    negl    %%edi\n \
    movl    0x40+var_24(%%esp),%%edx\n \
    movl    _gtblock_arr05(,%%edx,4),%%edx\n \
    imull   %%edi,%%edx\n \
    addl    %%edx,%%ebx\n \
    movl    0x40+var_30(%%esp),%%edx\n \
    imull   %%edi,%%edx\n \
    addl    %%edx,0x40+var_2C(%%esp)\n \
    movl    0x40+var_38(%%esp),%%edx\n \
    imull   %%edi,%%edx\n \
    addl    %%edx,0x40+var_34(%%esp)\n \
    xorl    %%edi,%%edi\n \
\n \
loc_7866D0:         #F5\n \
    movl    %%edi,%%edx\n \
    addl    %%eax,%%edx\n \
    cmpl    _LOC_gtblock_clip_width,%%edx\n \
    jle loc_7866FC\n \
    subl    %%edx,%%eax\n \
    addl    _LOC_gtblock_clip_width,%%eax\n \
    jz  locret1\n \
    js  locret1\n \
    movl    $8,%%edx\n \
    orl %%edx,0x40+var_28(%%esp)\n \
\n \
loc_7866FC:         #142\n \
    pushl   %%eax\n \
    movl    _LOC_gtblock_screen_width,%%eax\n \
    imull   %%eax,%%esi\n \
    addl    %%edi,%%esi\n \
    addl    _LOC_gtblock_screen_addr,%%esi\n \
    movl    0x44+var_2C(%%esp),%%ebp\n \
    movl    %%esi,%%eax\n \
    andl    $3,%%eax\n \
    addl    0x44+var_28(%%esp),%%eax\n \
    jmp     *gtblock_switch01(,%%eax,4)\n \
# ---------------------------------------------------------------------------\n \
\n \
gtb_case_04:         #182\n \
    pushl   %%ecx\n \
    pushl   %%esi\n \
    pushl   %%ebx\n \
    pushl   %%ebp\n \
    sarl    $8,%%ebp\n \
    movl    0x54+var_30(%%esp),%%edi\n \
    sarl    $8,%%edi\n \
    jns loc_786741\n \
    incl    %%edi\n \
\n \
loc_786741:         #1A6\n \
    movl    0x54+var_40(%%esp),%%ecx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,4(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,8(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x0C(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x10(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x14(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x18(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x1C(%%esi)\n \
    popl    %%ebp\n \
    popl    %%ebx\n \
    popl    %%esi\n \
    popl    %%ecx\n \
    movl    0x44+var_38(%%esp),%%eax\n \
    addl    %%eax,0x44+var_30(%%esp)\n \
    addl    0x44+var_34(%%esp),%%ebp\n \
    addl    0x44+var_3C(%%esp),%%ebx\n \
    addl    _LOC_gtblock_screen_width,%%esi\n \
    decl    %%ecx\n \
    jnz gtb_case_04\n \
    jmp locret2\n \
# ---------------------------------------------------------------------------\n \
\n \
gtb_case_05:         #182\n \
\
    pushl   %%ecx\n \
    pushl   %%esi\n \
    pushl   %%ebx\n \
    pushl   %%ebp\n \
    sarl    $8,%%ebp\n \
    movl    0x54+var_30(%%esp),%%edi\n \
    sarl    $8,%%edi\n \
    jns loc_786991\n \
    incl    %%edi\n \
\n \
loc_786991:         #3F6\n \
    movl    0x54+var_40(%%esp),%%ecx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,1(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,2(%%esi)\n \
    addl    $3,%%esi\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,4(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,8(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x0C(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x10(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x14(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x18(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,0x1C(%%esi)\n \
    popl    %%ebp\n \
    popl    %%ebx\n \
    popl    %%esi\n \
    popl    %%ecx\n \
    movl    0x44+var_38(%%esp),%%eax\n \
    addl    %%eax,0x44+var_30(%%esp)\n \
    addl    0x44+var_34(%%esp),%%ebp\n \
    addl    0x44+var_3C(%%esp),%%ebx\n \
    addl    _LOC_gtblock_screen_width,%%esi\n \
    decl    %%ecx\n \
    jnz gtb_case_05\n \
    jmp locret2\n \
# ---------------------------------------------------------------------------\n \
\n \
gtb_case_06:         #182\n \
    pushl   %%ecx\n \
    pushl   %%esi\n \
    pushl   %%ebx\n \
    pushl   %%ebp\n \
    sarl    $8,%%ebp\n \
    movl    0x54+var_30(%%esp),%%edi\n \
    sarl    $8,%%edi\n \
    jns loc_786BE1\n \
    incl    %%edi\n \
\n \
loc_786BE1:         #646\n \
    movl    0x54+var_40(%%esp),%%ecx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,1(%%esi)\n \
    addl    $2,%%esi\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,4(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,8(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x0C(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x10(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x14(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x18(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,0x1C(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,0x1D(%%esi)\n \
    popl    %%ebp\n \
    popl    %%ebx\n \
    popl    %%esi\n \
    popl    %%ecx\n \
    movl    0x44+var_38(%%esp),%%eax\n \
    addl    %%eax,0x44+var_30(%%esp)\n \
    addl    0x44+var_34(%%esp),%%ebp\n \
    addl    0x44+var_3C(%%esp),%%ebx\n \
    addl    _LOC_gtblock_screen_width,%%esi\n \
    decl    %%ecx\n \
    jnz gtb_case_06\n \
    jmp locret2\n \
# ---------------------------------------------------------------------------\n \
gtb_case_07:         #182\n \
    pushl   %%ecx\n \
    pushl   %%esi\n \
    pushl   %%ebx\n \
    pushl   %%ebp\n \
    sarl    $8,%%ebp\n \
    movl    0x54+var_30(%%esp),%%edi\n \
    sarl    $8,%%edi\n \
    jns loc_786E31\n \
    incl    %%edi\n \
\n \
loc_786E31:         #896\n \
    movl    0x54+var_40(%%esp),%%ecx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,(%%esi)\n \
    incl    %%esi\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,4(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,8(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x0C(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x10(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x14(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x18(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,0x1C(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,0x1D(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,0x1E(%%esi)\n \
    popl    %%ebp\n \
    popl    %%ebx\n \
    popl    %%esi\n \
    popl    %%ecx\n \
    movl    0x44+var_38(%%esp),%%eax\n \
    addl    %%eax,0x44+var_30(%%esp)\n \
    addl    0x44+var_34(%%esp),%%ebp\n \
    addl    0x44+var_3C(%%esp),%%ebx\n \
    addl    _LOC_gtblock_screen_width,%%esi\n \
    decl    %%ecx\n \
    jnz gtb_case_07\n \
    jmp locret2\n \
# ---------------------------------------------------------------------------\n \
gtb_case_00:         #182\n \
    pushl   %%ecx\n \
    pushl   %%esi\n \
    pushl   %%ebx\n \
    pushl   %%ebp\n \
    sarl    $8,%%ebp\n \
    movl    0x54+var_30(%%esp),%%edi\n \
    sarl    $8,%%edi\n \
    jns loc_787081\n \
    incl    %%edi\n \
\n \
loc_787081:         #AE6\n \
    movl    0x54+var_40(%%esp),%%ecx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,4(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,8(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,0x0C(%%esi)\n \
    popl    %%ebp\n \
    popl    %%ebx\n \
    popl    %%esi\n \
    popl    %%ecx\n \
    movl    0x44+var_38(%%esp),%%eax\n \
    addl    %%eax,0x44+var_30(%%esp)\n \
    addl    0x44+var_34(%%esp),%%ebp\n \
    addl    0x44+var_3C(%%esp),%%ebx\n \
    addl    _LOC_gtblock_screen_width,%%esi\n \
    decl    %%ecx\n \
    jnz gtb_case_00\n \
    jmp locret2\n \
# ---------------------------------------------------------------------------\n \
\n \
gtb_case_01:         #182\n \
# gtblock_draw_+D50j\n \
\
    pushl   %%ecx\n \
    pushl   %%esi\n \
    pushl   %%ebx\n \
    pushl   %%ebp\n \
    sarl    $8,%%ebp\n \
    movl    0x54+var_30(%%esp),%%edi\n \
    sarl    $8,%%edi\n \
    jns loc_7871C1\n \
    incl    %%edi\n \
\n \
loc_7871C1:         #C26\n \
    movl    0x54+var_40(%%esp),%%ecx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,1(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,2(%%esi)\n \
    addl    $3,%%esi\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,4(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,8(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,0x0C(%%esi)\n \
    popl    %%ebp\n \
    popl    %%ebx\n \
    popl    %%esi\n \
    popl    %%ecx\n \
    movl    0x44+var_38(%%esp),%%eax\n \
    addl    %%eax,0x44+var_30(%%esp)\n \
    addl    0x44+var_34(%%esp),%%ebp\n \
    addl    0x44+var_3C(%%esp),%%ebx\n \
    addl    _LOC_gtblock_screen_width,%%esi\n \
    decl    %%ecx\n \
    jnz gtb_case_01\n \
    jmp locret2\n \
# ---------------------------------------------------------------------------\n \
gtb_case_02:         #182\n \
# gtblock_draw_+EA0j\n \
\
    pushl   %%ecx\n \
    pushl   %%esi\n \
    pushl   %%ebx\n \
    pushl   %%ebp\n \
    sarl    $8,%%ebp\n \
    movl    0x54+var_30(%%esp),%%edi\n \
    sarl    $8,%%edi\n \
    jns loc_787311\n \
    incl    %%edi\n \
\n \
loc_787311:         #D76\n \
    movl    0x54+var_40(%%esp),%%ecx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,1(%%esi)\n \
    addl    $2,%%esi\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,4(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,8(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,0x0C(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,0x0D(%%esi)\n \
    popl    %%ebp\n \
    popl    %%ebx\n \
    popl    %%esi\n \
    popl    %%ecx\n \
    movl    0x44+var_38(%%esp),%%eax\n \
    addl    %%eax,0x44+var_30(%%esp)\n \
    addl    0x44+var_34(%%esp),%%ebp\n \
    addl    0x44+var_3C(%%esp),%%ebx\n \
    addl    _LOC_gtblock_screen_width,%%esi\n \
    decl    %%ecx\n \
    jnz gtb_case_02\n \
    jmp locret2\n \
# ---------------------------------------------------------------------------\n \
gtb_case_03:         #182\n \
    pushl   %%ecx\n \
    pushl   %%esi\n \
    pushl   %%ebx\n \
    pushl   %%ebp\n \
    sarl    $8,%%ebp\n \
    movl    0x54+var_30(%%esp),%%edi\n \
    sarl    $8,%%edi\n \
    jns loc_787461\n \
    incl    %%edi\n \
\n \
loc_787461:         #EC6\n \
    movl    0x54+var_40(%%esp),%%ecx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,(%%esi)\n \
    incl    %%esi\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,4(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    addl    %%edi,%%ebp\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%ah\n \
    popl    %%ebx\n \
    rorl    $0x10,%%eax\n \
    movl    %%eax,8(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,0x0C(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,0x0D(%%esi)\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    %%ecx,%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,0x0E(%%esi)\n \
    popl    %%ebp\n \
    popl    %%ebx\n \
    popl    %%esi\n \
    popl    %%ecx\n \
    movl    0x44+var_38(%%esp),%%eax\n \
    addl    %%eax,0x44+var_30(%%esp)\n \
    addl    0x44+var_34(%%esp),%%ebp\n \
    addl    0x44+var_3C(%%esp),%%ebx\n \
    addl    _LOC_gtblock_screen_width,%%esi\n \
    decl    %%ecx\n \
    jnz gtb_case_03\n \
    jmp locret2\n \
# ---------------------------------------------------------------------------\n \
loc_787590:         #182\n \
    pushl   %%ecx\n \
    pushl   %%esi\n \
    pushl   %%ebx\n \
    pushl   %%ebp\n \
    sarl    $8,%%ebp\n \
    movl    0x54+var_30(%%esp),%%edi\n \
    sarl    $8,%%edi\n \
    jns loc_7875A1\n \
    incl    %%edi\n \
\n \
loc_7875A1:         #1006\n \
    movl    0x54+var_44(%%esp),%%ecx\n \
\n \
loc_7875A8:         #1024j\n \
    movl    %%ebp,%%edx\n \
    movb    (%%ebx),%%dl\n \
    addl    0x54+var_40(%%esp),%%ebx\n \
    pushl   %%ebx\n \
    movl    _render_fade_tables,%%ebx\n \
    movb    (%%ebx,%%edx),%%al\n \
    popl    %%ebx\n \
    addl    %%edi,%%ebp\n \
    movb    %%al,(%%esi)\n \
    incl    %%esi\n \
    decl    %%ecx\n \
    jnz loc_7875A8\n \
    popl    %%ebp\n \
    popl    %%ebx\n \
    popl    %%esi\n \
    popl    %%ecx\n \
    movl    0x44+var_38(%%esp),%%eax\n \
    addl    %%eax,0x44+var_30(%%esp)\n \
    addl    0x44+var_34(%%esp),%%ebp\n \
    addl    0x44+var_3C(%%esp),%%ebx\n \
    addl    _LOC_gtblock_screen_width,%%esi\n \
    decl    %%ecx\n \
    jnz loc_787590\n \
    jmp locret2\n \
# ---------------------------------------------------------------------------\n \
gtblock_switch01:\n \
    .int    gtb_case_00\n \
    .int    gtb_case_01\n \
    .int    gtb_case_02\n \
    .int    gtb_case_03\n \
    .int    gtb_case_04\n \
    .int    gtb_case_05\n \
    .int    gtb_case_06\n \
    .int    gtb_case_07\n \
    .int    loc_787590\n \
    .int    loc_787590\n \
    .int    loc_787590\n \
    .int    loc_787590\n \
    .int    loc_787590\n \
    .int    loc_787590\n \
    .int    loc_787590\n \
    .int    loc_787590\n \
\n \
locret2:         #3D5\n \
    addl    $0x04,%%esp\n \
\n \
locret1:         #A1\n \
    addl    $0x20,%%esp\n \
    popa    \n \
"
         : // no outputs
         : "a" (gtb)
         : "memory", "cc");
#endif
}
/******************************************************************************/
